# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand

#' getInput2mvrnorm
#'
#' @inheritParams init_variable
#'
#' @return
#' a list that can be used as input for MASS::mvrnorm
#' @export
#'
#' @examples
#' list_var <- init_variable(name = "my_var", mu = 0, sd = 2, level = 3)
#' getInput2mvrnorm(list_var)
getInput2mvrnorm <- function(list_var){
  # -- pick up sd provided by user
  variable_standard_dev <- getGivenAttribute(list_var, attribute = "sd") %>% unlist()
  interaction_standard_dev <- getGivenAttribute(list_var$interactions, attribute = "sd") %>% unlist()
  list_stdev_2covmatx <- c(variable_standard_dev, interaction_standard_dev)
  if (is.null(list_stdev_2covmatx)) ## NO SD provided
    return(list(mu = NULL, covMatrix = NULL))

  # - COV matrix
  covar_userProvided = getGivenAttribute(list_var$correlations, "covar")
  covMatrix <- getCovarianceMatrix(list_stdev_2covmatx, covar_userProvided)

  # -- MU
  variable_mu <- getGivenAttribute(list_var, attribute = "mu") %>% unlist()
  interaction_mu <- getGivenAttribute(list_var$interactions, attribute = "mu") %>% unlist()
  list_mu <- c(variable_mu, interaction_mu)

  return(list(mu = list_mu, covMatrix = covMatrix))

}


#' getCovarianceMatrix 
#' @param list_stdev standard deviation list
#' @param list_covar covariance list
#' 
#' @return
#' covariance matrix
#' @export
#'
#' @examples
#' vector_sd <- c(1,2, 3)
#' names(vector_sd) <- c("varA", "varB", "varC")
#' vector_covar <- c(8, 12, 24)
#' names(vector_covar) <- c("varA.varB", "varA.varC", "varB.varC")
#' covMatrix <- getCovarianceMatrix(vector_sd, vector_covar)
getCovarianceMatrix <- function(list_stdev, list_covar){
  # -- cov(A, A) = sd(A)^2
  diag_cov <- list_stdev^2
  dimension <- length(diag_cov)
  covariance_matrix <- matrix(0,nrow = dimension, ncol = dimension)
  diag(covariance_matrix) <- diag_cov
  colnames(covariance_matrix) <- paste("label", names(diag_cov), sep = "_")
  rownames(covariance_matrix) <- paste("label", names(diag_cov), sep = "_")
  names_covaration <- names(list_covar)

  ###### -- utils -- #####
  convertDF <- function(name, value){
    ret <- data.frame(value)
    colnames(ret) <- name
    ret
  }

  ## -- needed to use reduce after ;)
  l_covarUserDf <- lapply(names_covaration, function(n_cov) convertDF(n_cov, list_covar[n_cov] ))
  covariance_matrix2ret <- Reduce(fillInCovarMatrice, x = l_covarUserDf, init =  covariance_matrix)
  covariance_matrix2ret
}


#' Fill in Covariance Matrix
#'
#' This function updates the covariance matrix with the specified covariance value between two variables.
#'
#' @param covarMatrice The input covariance matrix.
#' @param covar A data frame containing the covariance value between two variables.
#' @return The updated covariance matrix with the specified covariance value filled in.
#' @export
#' @examples
#' covarMat <- matrix(0, nrow = 3, ncol = 3)
#' colnames(covarMat) <- c("label_varA", "label_varB", "label_varC")
#' rownames(covarMat) <- c("label_varA", "label_varB", "label_varC")
#' covarValue <- data.frame("varA.varB" = 0.5)
#' fillInCovarMatrice(covarMatrice = covarMat, covar = covarValue)
fillInCovarMatrice <- function(covarMatrice, covar){
  varsInCovar <- strsplit(colnames(covar), split = "[.]") %>% unlist()
  index_matrix <- paste("label",varsInCovar, sep  = "_")
  covar_value <- covar[1,1]
  covarMatrice[index_matrix[1], index_matrix[2]] <- covar_value
  covarMatrice[index_matrix[2], index_matrix[1]] <- covar_value
  return(covarMatrice)
}

#' getGeneMetadata
#'
#' @inheritParams init_variable
#' @param n_genes Number of genes to simulate
#'
#' @return
#' metadata matrix
#' 
#' @export
#'
#' @examples
#' list_var <- init_variable()
#' metadata <- getGeneMetadata(list_var, n_genes = 10)
getGeneMetadata <- function(list_var, n_genes) {
  metaData <- generateGridCombination_fromListVar(list_var)
  n_combinations <- dim(metaData)[1]
  genes_vec <- paste("gene", 1:n_genes, sep = "")
  geneID <- rep(genes_vec, each = n_combinations)
  metaData <- cbind(geneID, metaData)
  
  return(metaData)
}


#' getDataFromMvrnorm
#'
#' @inheritParams init_variable 
#' @param input2mvrnorm list with mu and covariance matrix, output of getInput2mvrnorm
#' @param n_genes Number of genes to simulate
#' 
#' @return
#' data simulated from multivariate normal distribution
#' 
#' @export
#'
#' @examples
#' list_var <- init_variable()
#' input <- getInput2mvrnorm(list_var)
#' simulated_data <- getDataFromMvrnorm(list_var, input, n_genes = 10)
getDataFromMvrnorm <- function(list_var, input2mvrnorm, n_genes = 1) {
  if (is.null(input2mvrnorm$covMatrix))
    return(list())
  
  metaData <- getGeneMetadata(list_var, n_genes)
  n_tirages <- dim(metaData)[1]
  
  mtx_mvrnormSamplings <- samplingFromMvrnorm(n_samplings = n_tirages, 
                                             l_mu = input2mvrnorm$mu, matx_cov = input2mvrnorm$covMatrix)
  
  dataFromMvrnorm <- cbind(metaData, mtx_mvrnormSamplings)
  
  return(list(dataFromMvrnorm))
}


#' getDataFromMvrnorm
#'
#' @param n_samplings number of samplings using mvrnorm
#' @param l_mu vector of mu
#' @param matx_cov covariance matrix
#'
#' @return
#' samples generated from multivariate normal distribution
#' 
#' @export
#' @importFrom MASS mvrnorm
#' @examples
#' n <- 100
#' mu <- c(0, 0)
#' covMatrix <- matrix(c(1, 0.5, 0.5, 1), ncol = 2)
#' samples <- samplingFromMvrnorm(n_samplings = n, l_mu = mu, matx_cov = covMatrix)
samplingFromMvrnorm <- function(n_samplings, l_mu, matx_cov) {
  mvrnormSamp <-  MASS::mvrnorm(n = n_samplings, mu = l_mu, Sigma = matx_cov, empirical = TRUE)
  return(mvrnormSamp)
}

